// 	GUIDER_DECODER for DSP pcb version 2
// 	Author           	Greg Burley
//  Organization        OCIW
//  Revision Level      2.3
//  Target Device       EPM7128STC100-6

// Module Declaration
module gcam (D,Q,A,CS,CLK,RESET,WR,RD,AA3,IRQA,BWR,BRD,
	BA2,BA1,BA0,SQCK,SQEN,G0,G1,G2,G3,G4);
	
inout	[15:0]  D;				// bidirectional dsp data bus
inout	[15:0]	Q;				// bidirectional 'quiet bus' to peripherals
input	[7:0]	A;				// dsp address bus
output	[8:0] 	CS;				// epld generated chip selects for peripherals
input	CLK;					// not used
input	RESET;					// not used
input	WR, RD;					// dsp write and read
input	AA3;					// dsp external peripherals 
output	IRQA;
output	BWR, BRD;				// buffered backplane write + read signals
output	BA2, BA1, BA0;			// buffered backplane address lines
output	SQCK, SQEN;				// SEQuence register enable and clk lines
output	G0,G1,G2,G3,G4;			// spare pins

wire	OEa, OEb;				// bi-directional bus output enables

// 	SEQREG ADDRESS IS $FFFF80 USING /AA3
assign SQCK = WR;
assign SQEN = ~(~AA3 & (A[7:0]=='H80));

// 	SET UP THE TWO BI-DIRECTIONAL BUSES [D->Q for SEQREG and DACs,
//	Q->D for ADCs]
assign Q = (OEa) ? D : 16'bz;	// tristate outputs
assign D = (OEb) ? Q : 16'bz;
assign OEa = ~AA3 & ~WR & (A[7:0] >= 'H08);
assign OEb = ~AA3 & ~RD; 

// BUFFERED BACKPLANE SIGNALS
assign BWR = WR;
assign BRD = RD;
assign BA2 = A[2];
assign BA1 = A[1];
assign BA0 = A[0];

// INITIALIZATION SIGNAL FOR DSP PIN
assign IRQA = 1'b1;

//	CHIP SELECT		DECODED ADDRESS			DEVICE
//	===========		==================		=======
//	/CS0			$FFFF80					SEQ_REG	
//	/CS1			$FFFF81					ADC_A
//	/CS2			$FFFF82					ADC_B
//	/CS3			$FFFF88 -- $FFFF8F		DAC_1 [BIAS0..7]
//	/CS4			$FFFF90 -- $FFFF97		DAC_2 [V0..8]
//	/CS5			$FFFF98 -- $FFFF9F		DAC_3 [V9..15]

assign CS[0] = ~((A[7:0] == 'H80) & ~AA3 & ~WR);
assign CS[1] = ~((A[7:0] == 'H81) & ~AA3 & ~RD);
assign CS[2] = ~((A[7:0] == 'H82) & ~AA3 & ~RD);

assign CS[3] = ~((A[7:0] >= 'H88) & (A[7:0] <= 'H8F) & ~AA3);
assign CS[4] = ~((A[7:0] >= 'H90) & (A[7:0] <= 'H97) & ~AA3);
assign CS[5] = ~((A[7:0] >= 'H98) & (A[7:0] <= 'H9F) & ~AA3);

assign CS[6] = ~((A[7:0] == 'H83) & ~AA3 & ~RD);
assign CS[7] = ~((A[7:0] == 'H84) & ~AA3 & ~RD);
assign CS[8] = ~((A[7:0] >= 'HA0) & (A[7:0] <= 'HA7) & ~AA3);

//	EXTRA PINS FOR FUTURE USE (?)
assign G0 = 1'bz;
assign G1 = 1'bz;
assign G2 = 1'bz;
assign G3 = 1'bz;
assign G4 = 1'bz;

endmodule



